# Generated by Django 2.2.12 on 2020-05-10 19:59

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('discord', '0002_service_permissions'),
    ]

    operations = [
        migrations.AddField(
            model_name='discorduser',
            name='activated',
            field=models.DateTimeField(blank=True, default=None, help_text='Date & time this service account was activated', null=True),
        ),
        migrations.AddField(
            model_name='discorduser',
            name='discriminator',
            field=models.CharField(blank=True, default='', help_text="user's discriminator on Discord", max_length=4),
        ),
        migrations.AddField(
            model_name='discorduser',
            name='username',
            field=models.CharField(blank=True, db_index=True, default='', help_text="user's username on Discord", max_length=32),
        ),
        migrations.AlterField(
            model_name='discorduser',
            name='uid',
            field=models.BigIntegerField(db_index=True, help_text="user's ID on Discord"),
        ),
        migrations.AlterField(
            model_name='discorduser',
            name='user',
            field=models.OneToOneField(help_text='Auth user owning this Discord account', on_delete=django.db.models.deletion.CASCADE, primary_key=True, related_name='discord', serialize=False, to=settings.AUTH_USER_MODEL),
        ),
    ]
