{% extends "allianceauth/base.html" %}
{% load bootstrap %}
{% load static %}
{% load i18n %}

{% block page_title %}{% trans "Update AAR Link" %}{% endblock page_title %}

{% block content %}

    <div class="col-lg-12">
        <h1 class="page-header text-center">{% trans "Update AAR Link" %}</h1>

        <div class="container-fluid">
            <div class="col-md-4 col-md-offset-4">
                <div class="row">
                    {% if no_fleet_id %}
                        <div class="alert alert-danger" role="alert">{% trans "SRP Fleet Does Not Exist" %}</div>
                    {% else %}
                        <form class="form-signin" role="form" action="" method="POST">
                            {% csrf_token %}
                            {{ form|bootstrap }}
                            <br/>
                            <button class="btn btn-lg btn-primary btn-block" type="submit">{% trans "Update AAR Link" %}
                            </button>
                        </form>
                    {% endif %}
                </div>
            </div>
        </div>

    </div>

{% endblock content %}
