{% load static %}
<!-- Bootstrap CSS -->
{% if NIGHT_MODE %}
    {% if debug %}
        <!-- In template debug, loading less file instead of CSS -->
        <link rel="stylesheet/less" type="text/css" href="{% static 'css/themes/darkly/darkly.less' %}" />
        <script src="https://cdnjs.cloudflare.com/ajax/libs/less.js/2.7.3/less.min.js"></script>
    {% else %}
        <link rel="stylesheet" type="text/css" href="{% static 'css/themes/darkly/darkly.min.css' %}" />
    {% endif %}
{% else %}
    {% if debug %}
        <!-- In template debug, loading less file instead of CSS -->
        <link rel="stylesheet/less" type="text/css" href="{% static 'css/themes/flatly/flatly.less' %}" />
        <script src="https://cdnjs.cloudflare.com/ajax/libs/less.js/2.7.3/less.min.js"></script>
    {% else %}
        <link rel="stylesheet" type="text/css" href="{% static 'css/themes/flatly/flatly.min.css' %}" />
    {% endif %}
{% endif %}
<!-- End Bootstrap CSS -->
