<!-- Start jQuery-DateTimePicker CSS from cdnjs -->
<link rel="stylesheet" type="text/css" href="https://cdnjs.cloudflare.com/ajax/libs/jquery-datetimepicker/2.5.20/jquery.datetimepicker.min.css"/>
<!-- End jQuery-DateTimePicker CSS from cdnjs -->  