.gray-icon-color .fa {
    color: #505050;
}

.notification-bell-color {
    color: #A88F1E;
}

.navbar-brand {
    height: 58px;
    padding: 19px 19px;
}

.auth-navbar-top {
    margin-bottom: 0;
}

.auth-side-navbar-collapse {
    padding-left: 0;
}

#wrapper {
    width: 100%;
    padding: 0;
}

#site-body-wrapper {
    margin-right:0;
}

/* Horizontal list group */
ul.list-group.list-group-horizontal {
    display: table;
    width: 100%;
}

ul.list-group.list-group-horizontal > li.list-group-item {
    display: table-cell;
    max-width: 0;
}

.flex-center-horizontal {
    justify-content: center;
}

/* style group headers within a table */
.tr-group {
    font-weight: bold;
    background-color: #e6e6e6 !important;
}

/* default style for tables */
.table-aa > thead > tr > th{
    border-bottom: 1px solid #f2f2f2;
}
.table-aa > thead > tr > th{
    vertical-align: middle;
}
.table-aa > tbody > tr > td{
    border-bottom: 1px solid #f2f2f2;
}
.table-aa > tbody > tr > td {
    vertical-align: middle;
}
.table-aa > tbody > tr:last-child {
    border-bottom: none;
}

/* highlight active menu items
--------------------------------------------------------------------------------------------------------------------- */
@media all {
    .template-light-mode .nav-pills > li > a.active {
        background-color: rgb(236, 240, 241);
    }

    .template-dark-mode .nav-pills > li > a.active {
        background-color: rgb(48, 48, 48);
    }
}

/* Small devices (tablets, 768px and up) */
@media (min-width: 768px) {

    /* class for vertically aligning columns in a bootstrap row */
    .row.vertical-flexbox-row2 {
        display:  -webkit-box;
        display: -webkit-flex;
        display:  -ms-flexbox;
        display:         flex;
        flex-wrap: wrap;
      }

      .row.vertical-flexbox-row2 > [class*='col-'] {
        display: flex;
        flex-direction: column;
      }
}

/* Extra Small devices (Phones, <768px) */
@media (max-width: 767px) {
    .button-wrapper .btn {
        margin-bottom:5px;
    }
}
