<!-- Start jQuery-DateTimePicker JS from cdnjs -->
<script type="application/javascript" src="https://cdnjs.cloudflare.com/ajax/libs/jquery-datetimepicker/2.5.20/jquery.datetimepicker.full.min.js"></script>
<!-- End jQuery-DateTimePicker JS from cdnjs -->  