# Generated by Django 2.2.9 on 2020-03-16 07:49

from django.db import migrations, models

class Migration(migrations.Migration):

    dependencies = [
        ('mumble', '0007_not_null_user'),
    ]

    operations = [
        migrations.AddField(
            model_name='mumbleuser',
            name='display_name',
            field=models.CharField(max_length=254, null=True),
        )
    ]
