# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-13 04:42
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('optimer', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='optimer',
            name='details',
        ),
        migrations.RemoveField(
            model_name='optimer',
            name='location',
        ),
    ]
