# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-04 07:17
from __future__ import unicode_literals

from django.db import migrations
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.management import create_permissions

import logging

logger = logging.getLogger(__name__)


class Migration(migrations.Migration):

    dependencies = [
        ('groupmanagement', '0005_authgroup_public'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='authgroup',
            options={'permissions': (('request_groups', 'Can request non-public groups'),)},
        ),
    ]
