# Generated by Django 2.0.4 on 2018-04-17 20:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0009_on_delete'),
    ]

    operations = [
        migrations.AddField(
            model_name='evecharacter',
            name='alliance_ticker',
            field=models.CharField(blank=True, default='', max_length=5, null=True),
        ),
        migrations.AlterField(
            model_name='evecharacter',
            name='corporation_ticker',
            field=models.CharField(max_length=5),
        ),
    ]
