{% extends "allianceauth/base.html" %}
{% load bootstrap %}
{% load static %}
{% load i18n %}

{% block page_title %}{% trans "Verify Teamspeak" %}{% endblock page_title %}
{% block extra_css %}{% endblock extra_css %}

{% block content %}
    <div class="col-lg-12">
        <h1 class="page-header text-center">{% trans "Verify Teamspeak Identity" %}</h1>
        <div class="container-fluid">
            <div class="col-md-4 col-md-offset-4">
                <a href="ts3server://{{ public_url }}?token={{ authinfo.teamspeak3_perm_key }}&nickname={{ authinfo.teamspeak3_uid }}" class="btn btn-primary btn-block btn-lg" title="Join">{% trans "Join Server" %}</a>
                <br/>
                <form class="form-signin" role="form" action="{% url 'teamspeak3:verify' %}" method="POST">
                    {% csrf_token %}
                    {{ form|bootstrap }}
                    <br/>
                    <button class="btn btn-lg btn-primary btn-block" type="submit">{% trans "Continue" %}</button>
                </form>
                </div>
        </div>
    </div>
{% endblock %}
