{% load i18n %}
{% load navactive %}
{% load menu_items %}
    
<div class="col-sm-2 auth-side-navbar" role="navigation">
    <div class="collapse navbar-collapse auth-menus-collapse auth-side-navbar-collapse">
        <ul class="nav nav-pills nav-stacked gray-icon-color" id="side-menu">
            <li>
                <a class="{% navactive request 'authentication:dashboard' %}"
                   href="{% url 'authentication:dashboard' %}">
                    <i class="fas fa-tachometer-alt fa-fw"></i> {% trans "Dashboard" %}
                </a>
            </li>
            <li>
                <a class="{% navactive request 'groupmanagement:groups' %}" href="{% url 'groupmanagement:groups' %}">
                    <i class="fas fa-users fa-fw"></i> {% trans "Groups" %}
                </a>
            </li>           

            {% menu_items %}
           
        </ul>
    </div>
</div>
