{% load i18n %}{% autoescape off %}
    {% blocktrans trimmed %}You're receiving this email because you requested a password reset for your
        user account.{% endblocktrans %}

    {% trans "Please go to the following page and choose a new password:" %}
    {% block reset_link %}
        {{domain}}{% url 'password_reset_confirm' uidb64=uid token=token %}
    {% endblock %}
    {% trans "Your username, in case you've forgotten:" %} {{ user.get_username }}

    {% trans "Thanks for using our site!" %}

    {% blocktrans %}Your IT Team{% endblocktrans %}

{% endautoescape %}
