{% load static %}
{% load i18n %}
{% load navactive %}

<nav class="navbar navbar-default">
    <div class="container-fluid">

        <div class="navbar-header">
            <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#bs-example-navbar-collapse-1" aria-expanded="false">
                <span class="sr-only">{% trans "Toggle navigation" %}</span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
            </button>
            <a class="navbar-brand" href="">{% trans "Group Management" %}</a>
        </div>

        <div class="collapse navbar-collapse" id="bs-example-navbar-collapse-1">
            <ul class="nav navbar-nav">
                <li class="{% navactive request 'groupmanagement:management' %}">
                    <a href="{% url 'groupmanagement:management' %}">{% trans "Group Requests" %}</a>
                </li>
                <li class="{% renavactive request '^/group/membership/' %}">
                    <a href="{% url 'groupmanagement:membership' %}">{% trans "Group Membership" %}</a>
                </li>
            </ul>
        </div>
        
    </div>
</nav>
