{% extends "allianceauth/base.html" %}
{% load bootstrap %}
{% load static %}
{% load i18n %}
{% get_current_language as LANGUAGE_CODE %}

{% block page_title %}{% trans "Create Operation" %}{% endblock page_title %}
{% block extra_css %}
    {% include 'bundles/jquery-datetimepicker-css.html' %}
{% endblock extra_css %}

{% block content %}
    <div class="col-lg-12">
        <h1 class="page-header text-center">{% trans "Create Fleet Operation" %}</h1>

        <div class="container-fluid">
            <div class="col-md-4 col-md-offset-4">
                <div class="row">
                        <form class="form-signin" role="form" action="" method="POST">
                            {% csrf_token %}
                            {{ form|bootstrap }}
                            <br/>
                            <button class="btn btn-lg btn-primary btn-block" type="submit">{% trans "Create Fleet Operation" %}</button>
                        </form>
                </div>
            </div>
        </div>
    </div>

{% endblock content %}

{% block extra_javascript %}
    {% include 'bundles/jquery-datetimepicker-js.html' %}
{% endblock %}

{% block extra_script %}

    $('#id_start').datetimepicker({
    setlocale: '{{ LANGUAGE_CODE }}',
    {% if NIGHT_MODE %}
        theme: 'dark',
    {% else %}
        theme: 'default',
    {% endif %}
    mask: true,
    format: 'Y-m-d H:i',
    minDate: 0
    });

{% endblock extra_script %}
